@@ -27,16 +27,16 @@ from __future__ import annotations
 from alembic import op
 
 # revision identifiers, used by Alembic.
-revision = '211e584da130'
-down_revision = '2e82aab8ef20'
+revision = "211e584da130"
+down_revision = "2e82aab8ef20"
 branch_labels = None
 depends_on = None
-airflow_version = '1.7.1.3'
+airflow_version = "1.7.1.3"
 
 
 def upgrade():
-    op.create_index('ti_state', 'task_instance', ['state'], unique=False)
+    op.create_index("ti_state", "task_instance", ["state"], unique=False)
 
 
 def downgrade():
-    op.drop_index('ti_state', table_name='task_instance')
+    op.drop_index("ti_state", table_name="task_instance")
