@@ -18,11 +18,13 @@ from __future__ import annotations
 
 from airflow.sensors.base import BaseSensorOperator
 
+# Ignore missing docstring
+
 _EXCEPTION_MESSAGE = """The old HDFS Sensors have been removed in 4.0.0 version of the apache.hdfs provider.
 Please convert your DAGs to use the WebHdfsSensor or downgrade the provider to below 4.*
 if you want to continue using it.
 If you want to use earlier provider you can downgrade to latest released 3.* version
-using `pip install apache-airflow-providers-hdfs==3.2.1` (no constraints)
+using `pip install apache-airflow-providers-apache-hdfs==3.2.1` (no constraints)
 """
 
 
@@ -33,7 +35,7 @@ class HdfsSensor(BaseSensorOperator):
     Please convert your DAGs to use the WebHdfsSensor or downgrade the provider
     to below 4.* if you want to continue using it. If you want to use earlier
     provider you can downgrade to latest released 3.* version using
-    `pip install apache-airflow-providers-hdfs==3.2.1` (no constraints).
+    `pip install apache-airflow-providers-apache-hdfs==3.2.1` (no constraints).
     """
 
     def __init__(self, *args, **kwargs):
