@@ -19,8 +19,9 @@ KubernetesExecutor.
 
 .. seealso::
     For more information on how the KubernetesExecutor works, take a look at the guide:
-    :ref:`executor:KubernetesExecutor`
+    :doc:`/kubernetes_executor`
 """
+
 from __future__ import annotations
 
 import contextlib
@@ -28,12 +29,13 @@ import json
 import logging
 import multiprocessing
 import time
-from collections import defaultdict
+from collections import Counter, defaultdict
 from contextlib import suppress
 from datetime import datetime
 from queue import Empty, Queue
 from typing import TYPE_CHECKING, Any, Sequence
 
+from kubernetes.dynamic import DynamicClient
 from sqlalchemy import select, update
 
 from airflow.providers.cncf.kubernetes.pod_generator import PodMutationHookException, PodReconciliationError
@@ -74,7 +76,10 @@ except ImportError:
     raise
 from airflow.configuration import conf
 from airflow.executors.base_executor import BaseExecutor
-from airflow.providers.cncf.kubernetes.executors.kubernetes_executor_types import POD_EXECUTOR_DONE_KEY
+from airflow.providers.cncf.kubernetes.executors.kubernetes_executor_types import (
+    ADOPTED,
+    POD_EXECUTOR_DONE_KEY,
+)
 from airflow.providers.cncf.kubernetes.kube_config import KubeConfig
 from airflow.providers.cncf.kubernetes.kubernetes_helper_functions import annotations_to_key
 from airflow.utils.event_scheduler import EventScheduler
@@ -157,22 +162,36 @@ class KubernetesExecutor(BaseExecutor):
         self.event_scheduler: EventScheduler | None = None
         self.last_handled: dict[TaskInstanceKey, float] = {}
         self.kubernetes_queue: str | None = None
+        self.task_publish_retries: Counter[TaskInstanceKey] = Counter()
+        self.task_publish_max_retries = conf.getint("kubernetes", "task_publish_max_retries", fallback=0)
         super().__init__(parallelism=self.kube_config.parallelism)
 
     def _list_pods(self, query_kwargs):
+        query_kwargs["header_params"] = {
+            "Accept": "application/json;as=PartialObjectMetadataList;v=v1;g=meta.k8s.io"
+        }
+        dynamic_client = DynamicClient(self.kube_client.api_client)
+        pod_resource = dynamic_client.resources.get(api_version="v1", kind="Pod")
         if self.kube_config.multi_namespace_mode:
             if self.kube_config.multi_namespace_mode_namespace_list:
-                pods = []
-                for namespace in self.kube_config.multi_namespace_mode_namespace_list:
-                    pods.extend(
-                        self.kube_client.list_namespaced_pod(namespace=namespace, **query_kwargs).items
-                    )
+                namespaces = self.kube_config.multi_namespace_mode_namespace_list
             else:
-                pods = self.kube_client.list_pod_for_all_namespaces(**query_kwargs).items
+                namespaces = [None]
         else:
-            pods = self.kube_client.list_namespaced_pod(
-                namespace=self.kube_config.kube_namespace, **query_kwargs
-            ).items
+            namespaces = [self.kube_config.kube_namespace]
+
+        pods = []
+        for namespace in namespaces:
+            # Dynamic Client list pods is throwing TypeError when there are no matching pods to return
+            # This bug was fixed in MR https://github.com/kubernetes-client/python/pull/2155
+            # TODO: Remove the try-except clause once we upgrade the K8 Python client version which
+            # includes the above MR
+            try:
+                pods.extend(
+                    dynamic_client.get(resource=pod_resource, namespace=namespace, **query_kwargs).items
+                )
+            except TypeError:
+                continue
 
         return pods
 
@@ -409,28 +428,39 @@ class KubernetesExecutor(BaseExecutor):
                 task = self.task_queue.get_nowait()
 
                 try:
+                    key, command, kube_executor_config, pod_template_file = task
                     self.kube_scheduler.run_next(task)
+                    self.task_publish_retries.pop(key, None)
                 except PodReconciliationError as e:
-                    self.log.error(
+                    self.log.exception(
                         "Pod reconciliation failed, likely due to kubernetes library upgrade. "
                         "Try clearing the task to re-run.",
-                        exc_info=True,
                     )
                     self.fail(task[0], e)
                 except ApiException as e:
-                    # These codes indicate something is wrong with pod definition; otherwise we assume pod
-                    # definition is ok, and that retrying may work
-                    if e.status in (400, 422):
-                        self.log.error("Pod creation failed with reason %r. Failing task", e.reason)
-                        key, _, _, _ = task
-                        self.change_state(key, TaskInstanceState.FAILED, e)
-                    else:
+                    body = json.loads(e.body)
+                    retries = self.task_publish_retries[key]
+                    # In case of exceeded quota errors, requeue the task as per the task_publish_max_retries
+                    if (
+                        str(e.status) == "403"
+                        and "exceeded quota" in body["message"]
+                        and (self.task_publish_max_retries == -1 or retries < self.task_publish_max_retries)
+                    ):
                         self.log.warning(
-                            "ApiException when attempting to run task, re-queueing. Reason: %r. Message: %s",
+                            "[Try %s of %s] Kube ApiException for Task: (%s). Reason: %r. Message: %s",
+                            self.task_publish_retries[key] + 1,
+                            self.task_publish_max_retries,
+                            key,
                             e.reason,
-                            json.loads(e.body)["message"],
+                            body["message"],
                         )
                         self.task_queue.put(task)
+                        self.task_publish_retries[key] = retries + 1
+                    else:
+                        self.log.error("Pod creation failed with reason %r. Failing task", e.reason)
+                        key, _, _, _ = task
+                        self.fail(key, e)
+                        self.task_publish_retries.pop(key, None)
                 except PodMutationHookException as e:
                     key, _, _, _ = task
                     self.log.error(
@@ -450,7 +480,7 @@ class KubernetesExecutor(BaseExecutor):
     def _change_state(
         self,
         key: TaskInstanceKey,
-        state: TaskInstanceState | None,
+        state: TaskInstanceState | str | None,
         pod_name: str,
         namespace: str,
         session: Session = NEW_SESSION,
@@ -458,6 +488,15 @@ class KubernetesExecutor(BaseExecutor):
         if TYPE_CHECKING:
             assert self.kube_scheduler
 
+        if state == ADOPTED:
+            # When the task pod is adopted by another executor,
+            # then remove the task from the current executor running queue.
+            try:
+                self.running.remove(key)
+            except KeyError:
+                self.log.debug("TI key not in running: %s", key)
+            return
+
         if state == TaskInstanceState.RUNNING:
             self.event_buffer[key] = state, None
             return
@@ -480,7 +519,7 @@ class KubernetesExecutor(BaseExecutor):
             from airflow.models.taskinstance import TaskInstance
 
             state = session.scalar(select(TaskInstance.state).where(TaskInstance.filter_for_tis([key])))
-            state = TaskInstanceState(state)
+            state = TaskInstanceState(state) if state else None
 
         self.event_buffer[key] = state, None
 
