@@ -45,7 +45,11 @@ from googleapiclient.discovery import Resource, build
 from googleapiclient.errors import HttpError
 
 from airflow.exceptions import AirflowException, AirflowProviderDeprecationWarning
-from airflow.providers.google.common.hooks.base_google import GoogleBaseAsyncHook, GoogleBaseHook
+from airflow.providers.google.common.hooks.base_google import (
+    PROVIDE_PROJECT_ID,
+    GoogleBaseAsyncHook,
+    GoogleBaseHook,
+)
 
 if TYPE_CHECKING:
     from google.cloud.storage_transfer_v1.services.storage_transfer_service.pagers import (
@@ -250,7 +254,11 @@ class CloudDataTransferServiceHook(GoogleBaseHook):
                 request_filter = kwargs["filter"]
                 if not isinstance(request_filter, dict):
                     raise ValueError(f"The request_filter should be dict and is {type(request_filter)}")
-                warnings.warn("Use 'request_filter' instead of 'filter'", AirflowProviderDeprecationWarning)
+                warnings.warn(
+                    "Use 'request_filter' instead of 'filter'",
+                    AirflowProviderDeprecationWarning,
+                    stacklevel=2,
+                )
             else:
                 raise TypeError("list_transfer_job missing 1 required positional argument: 'request_filter'")
 
@@ -374,7 +382,11 @@ class CloudDataTransferServiceHook(GoogleBaseHook):
                 request_filter = kwargs["filter"]
                 if not isinstance(request_filter, dict):
                     raise ValueError(f"The request_filter should be dict and is {type(request_filter)}")
-                warnings.warn("Use 'request_filter' instead of 'filter'", AirflowProviderDeprecationWarning)
+                warnings.warn(
+                    "Use 'request_filter' instead of 'filter'",
+                    AirflowProviderDeprecationWarning,
+                    stacklevel=2,
+                )
             else:
                 raise TypeError(
                     "list_transfer_operations missing 1 required positional argument: 'request_filter'"
@@ -495,14 +507,14 @@ class CloudDataTransferServiceHook(GoogleBaseHook):
 class CloudDataTransferServiceAsyncHook(GoogleBaseAsyncHook):
     """Asynchronous hook for Google Storage Transfer Service."""
 
-    def __init__(self, project_id: str | None = None, **kwargs: Any) -> None:
+    def __init__(self, project_id: str = PROVIDE_PROJECT_ID, **kwargs: Any) -> None:
         super().__init__(**kwargs)
         self.project_id = project_id
         self._client: StorageTransferServiceAsyncClient | None = None
 
     def get_conn(self) -> StorageTransferServiceAsyncClient:
         """
-        Returns async connection to the Storage Transfer Service.
+        Return async connection to the Storage Transfer Service.
 
         :return: Google Storage Transfer asynchronous client.
         """
@@ -512,7 +524,7 @@ class CloudDataTransferServiceAsyncHook(GoogleBaseAsyncHook):
 
     async def get_jobs(self, job_names: list[str]) -> ListTransferJobsAsyncPager:
         """
-        Gets the latest state of a long-running operations in Google Storage Transfer Service.
+        Get the latest state of a long-running operations in Google Storage Transfer Service.
 
         :param job_names: (Required) List of names of the jobs to be fetched.
         :return: Object that yields Transfer jobs.
@@ -525,7 +537,7 @@ class CloudDataTransferServiceAsyncHook(GoogleBaseAsyncHook):
 
     async def get_latest_operation(self, job: TransferJob) -> Message | None:
         """
-        Gets the latest operation of the given TransferJob instance.
+        Get the latest operation of the given TransferJob instance.
 
         :param job: Transfer job instance.
         :return: The latest job operation.
