@@ -27,20 +27,18 @@ from flask import request, url_for
 from flask.helpers import flash
 from flask_appbuilder.forms import FieldConverter
 from flask_appbuilder.models.filters import BaseFilter
-from flask_appbuilder.models.sqla import Model, filters as fab_sqlafilters
+from flask_appbuilder.models.sqla import filters as fab_sqlafilters
 from flask_appbuilder.models.sqla.filters import get_field_setup_query, set_value_to_type
 from flask_appbuilder.models.sqla.interface import SQLAInterface
 from flask_babel import lazy_gettext
 from markdown_it import MarkdownIt
 from markupsafe import Markup
-from pendulum.datetime import DateTime
 from pygments import highlight, lexers
 from pygments.formatters import HtmlFormatter
-from pygments.lexer import Lexer
 from sqlalchemy import delete, func, select, types
 from sqlalchemy.ext.associationproxy import AssociationProxy
-from sqlalchemy.sql import Select
 
+from airflow.configuration import conf
 from airflow.exceptions import RemovedInAirflow3Warning
 from airflow.models import errors
 from airflow.models.dagrun import DagRun
@@ -52,14 +50,20 @@ from airflow.utils.helpers import alchemy_to_dict
 from airflow.utils.json import WebEncoder
 from airflow.utils.sqlalchemy import tuple_in_condition
 from airflow.utils.state import State, TaskInstanceState
+from airflow.www.extensions.init_auth_manager import get_auth_manager
 from airflow.www.forms import DateTimeWithTimezoneField
 from airflow.www.widgets import AirflowDateTimePickerWidget
 
 if TYPE_CHECKING:
+    from flask_appbuilder.models.sqla import Model
+    from pendulum.datetime import DateTime
+    from pygments.lexer import Lexer
     from sqlalchemy.orm.session import Session
+    from sqlalchemy.sql import Select
     from sqlalchemy.sql.operators import ColumnOperators
 
-    from airflow.www.fab_security.sqla.manager import SecurityManager
+    from airflow.www.extensions.init_appbuilder import AirflowAppBuilder
+
 
 TI = TaskInstance
 
@@ -107,7 +111,6 @@ priority: list[None | TaskInstanceState] = [
     TaskInstanceState.SCHEDULED,
     TaskInstanceState.DEFERRED,
     TaskInstanceState.RUNNING,
-    TaskInstanceState.SHUTDOWN,
     TaskInstanceState.RESTARTING,
     None,
     TaskInstanceState.SUCCESS,
@@ -152,16 +155,16 @@ def get_mapped_summary(parent_instance, task_instances):
 def get_dag_run_conf(
     dag_run_conf: Any, *, json_encoder: type[json.JSONEncoder] = json.JSONEncoder
 ) -> tuple[str | None, bool]:
-    conf: str | None = None
+    result: str | None = None
 
     conf_is_json: bool = False
     if isinstance(dag_run_conf, str):
-        conf = dag_run_conf
+        result = dag_run_conf
     elif isinstance(dag_run_conf, (dict, list)) and any(dag_run_conf):
-        conf = json.dumps(dag_run_conf, sort_keys=True, cls=json_encoder, ensure_ascii=False)
+        result = json.dumps(dag_run_conf, sort_keys=True, cls=json_encoder, ensure_ascii=False)
         conf_is_json = True
 
-    return conf, conf_is_json
+    return result, conf_is_json
 
 
 def encode_dag_run(
@@ -170,7 +173,7 @@ def encode_dag_run(
     if not dag_run:
         return None
 
-    conf, conf_is_json = get_dag_run_conf(dag_run.conf, json_encoder=json_encoder)
+    dag_run_conf, conf_is_json = get_dag_run_conf(dag_run.conf, json_encoder=json_encoder)
 
     return {
         "run_id": dag_run.run_id,
@@ -184,7 +187,7 @@ def encode_dag_run(
         "run_type": dag_run.run_type,
         "last_scheduling_decision": datetime_to_string(dag_run.last_scheduling_decision),
         "external_trigger": dag_run.external_trigger,
-        "conf": conf,
+        "conf": dag_run_conf,
         "conf_is_json": conf_is_json,
         "note": dag_run.note,
     }
@@ -251,7 +254,7 @@ def generate_pages(
     sorting_direction=None,
 ):
     """
-    Generates the HTML for a paging component.
+    Generate the HTML for a paging component.
 
     Uses a similar logic to the paging auto-generated by Flask managed views. The paging
     component defines a number of pages visible in the pager (window) and once the user
@@ -339,7 +342,7 @@ def generate_pages(
 
     output.append(previous_node.format(href_link=page_link, disabled=is_disabled))
 
-    mid = int(window / 2)
+    mid = window // 2
     last_page = num_of_pages - 1
 
     if current_page <= mid or num_of_pages < window:
@@ -404,37 +407,48 @@ def generate_pages(
 
 
 def epoch(dttm):
-    """Returns an epoch-type date (tuple with no timezone)."""
+    """Return an epoch-type date (tuple with no timezone)."""
     return (int(time.mktime(dttm.timetuple())) * 1000,)
 
 
 def make_cache_key(*args, **kwargs):
-    """Used by cache to get a unique key per URL."""
+    """Get a unique key per URL; used by cache."""
     path = request.path
     args = str(hash(frozenset(request.args.items())))
     return (path + args).encode("ascii", "ignore")
 
 
 def task_instance_link(attr):
-    """Generates a URL to the Graph view for a TaskInstance."""
+    """Generate a URL to the Graph view for a TaskInstance."""
     dag_id = attr.get("dag_id")
     task_id = attr.get("task_id")
-    execution_date = attr.get("dag_run.execution_date") or attr.get("execution_date") or timezone.utcnow()
+    run_id = attr.get("run_id")
+    map_index = attr.get("map_index", None)
+    if map_index == -1:
+        map_index = None
+
     url = url_for(
-        "Airflow.task",
+        "Airflow.grid",
         dag_id=dag_id,
         task_id=task_id,
-        execution_date=execution_date.isoformat(),
-        map_index=attr.get("map_index", -1),
+        dag_run_id=run_id,
+        map_index=map_index,
+        tab="graph",
     )
     url_root = url_for(
-        "Airflow.graph", dag_id=dag_id, root=task_id, execution_date=execution_date.isoformat()
+        "Airflow.grid",
+        dag_id=dag_id,
+        task_id=task_id,
+        root=task_id,
+        dag_run_id=run_id,
+        map_index=map_index,
+        tab="graph",
     )
     return Markup(
         """
         <span style="white-space: nowrap;">
         <a href="{url}">{task_id}</a>
-        <a href="{url_root}" title="Filter on this task and upstream">
+        <a href="{url_root}" title="Filter on this task">
         <span class="material-icons" style="margin-left:0;"
             aria-hidden="true">filter_alt</span>
         </a>
@@ -444,7 +458,7 @@ def task_instance_link(attr):
 
 
 def state_token(state):
-    """Returns a formatted string with HTML for a given State."""
+    """Return a formatted string with HTML for a given State."""
     color = State.color(state)
     fg_color = State.color_fg(state)
     return Markup(
@@ -456,13 +470,13 @@ def state_token(state):
 
 
 def state_f(attr):
-    """Gets 'state' & returns a formatted string with HTML for a given State."""
+    """Get 'state' & return a formatted string with HTML for a given State."""
     state = attr.get("state")
     return state_token(state)
 
 
 def nobr_f(attr_name):
-    """Returns a formatted string with HTML with a Non-breaking Text element."""
+    """Return a formatted string with HTML with a Non-breaking Text element."""
 
     def nobr(attr):
         f = attr.get(attr_name)
@@ -472,7 +486,7 @@ def nobr_f(attr_name):
 
 
 def datetime_f(attr_name):
-    """Returns a formatted string with HTML for given DataTime."""
+    """Return a formatted string with HTML for given DataTime."""
 
     def dt(attr):
         f = attr.get(attr_name)
@@ -494,7 +508,7 @@ def datetime_html(dttm: DateTime | None) -> str:
 
 
 def json_f(attr_name):
-    """Returns a formatted string with HTML for given JSON serializable."""
+    """Return a formatted string with HTML for given JSON serializable."""
 
     def json_(attr):
         f = attr.get(attr_name)
@@ -505,7 +519,7 @@ def json_f(attr_name):
 
 
 def dag_link(attr):
-    """Generates a URL to the Graph view for a Dag."""
+    """Generate a URL to the Graph view for a Dag."""
     dag_id = attr.get("dag_id")
     execution_date = attr.get("execution_date")
     if not dag_id:
@@ -515,11 +529,16 @@ def dag_link(attr):
 
 
 def dag_run_link(attr):
-    """Generates a URL to the Graph view for a DagRun."""
+    """Generate a URL to the Graph view for a DagRun."""
     dag_id = attr.get("dag_id")
     run_id = attr.get("run_id")
-    execution_date = attr.get("dag_run.execution_date") or attr.get("execution_date")
-    url = url_for("Airflow.graph", dag_id=dag_id, run_id=run_id, execution_date=execution_date)
+
+    url = url_for(
+        "Airflow.grid",
+        dag_id=dag_id,
+        dag_run_id=run_id,
+        tab="graph",
+    )
     return Markup('<a href="{url}">{run_id}</a>').format(url=url, run_id=run_id)
 
 
@@ -611,7 +630,7 @@ def json_render(obj, lexer):
 
 def wrapped_markdown(s, css_class="rich_doc"):
     """Convert a Markdown string to HTML."""
-    md = MarkdownIt("gfm-like")
+    md = MarkdownIt("gfm-like", {"html": conf.getboolean("webserver", "allow_raw_html_descriptions")})
     if s is None:
         return None
     s = textwrap.dedent(s)
@@ -645,18 +664,6 @@ def get_attr_renderer():
     }
 
 
-def get_chart_height(dag):
-    """
-    Use the number of tasks in the DAG to approximate the size of generated chart.
-
-    Without this the charts are tiny and unreadable when DAGs have a large number of tasks).
-    Ideally nvd3 should allow for dynamic-height charts, that is charts that take up space
-    based on the size of the components within.
-    TODO(aoen): See [AIRFLOW-1263].
-    """
-    return 600 + len(dag.tasks) * 10
-
-
 class UtcAwareFilterMixin:
     """Mixin for filter for UTC time."""
 
@@ -775,12 +782,13 @@ class AirflowFilterConverter(fab_sqlafilters.SQLAFilterConverter):
             "is_extendedjson",
             [],
         ),
-    ) + fab_sqlafilters.SQLAFilterConverter.conversion_table
+        *fab_sqlafilters.SQLAFilterConverter.conversion_table,
+    )
 
     def __init__(self, datamodel):
         super().__init__(datamodel)
 
-        for method, filters in self.conversion_table:
+        for _, filters in self.conversion_table:
             if FilterIsNull not in filters:
                 filters.append(FilterIsNull)
             if FilterIsNotNull not in filters:
@@ -806,12 +814,11 @@ class CustomSQLAInterface(SQLAInterface):
         clean_column_names()
         # Support for AssociationProxy in search and list columns
         for obj_attr, desc in self.obj.__mapper__.all_orm_descriptors.items():
-            if not isinstance(desc, AssociationProxy):
-                continue
-            proxy_instance = getattr(self.obj, obj_attr)
-            if hasattr(proxy_instance.remote_attr.prop, "columns"):
-                self.list_columns[obj_attr] = proxy_instance.remote_attr.prop.columns[0]
-                self.list_properties[obj_attr] = proxy_instance.remote_attr.prop
+            if isinstance(desc, AssociationProxy):
+                proxy_instance = getattr(self.obj, obj_attr)
+                if hasattr(proxy_instance.remote_attr.prop, "columns"):
+                    self.list_columns[obj_attr] = proxy_instance.remote_attr.prop.columns[0]
+                    self.list_properties[obj_attr] = proxy_instance.remote_attr.prop
 
     def is_utcdatetime(self, col_name):
         """Check if the datetime is a UTC one."""
@@ -827,7 +834,7 @@ class CustomSQLAInterface(SQLAInterface):
         return False
 
     def is_extendedjson(self, col_name):
-        """Checks if it is a special extended JSON type."""
+        """Check if it is a special extended JSON type."""
         from airflow.utils.sqlalchemy import ExtendedJSON
 
         if col_name in self.list_columns:
@@ -877,7 +884,8 @@ class DagRunCustomSQLAInterface(CustomSQLAInterface):
 # place
 FieldConverter.conversion_table = (
     ("is_utcdatetime", DateTimeWithTimezoneField, AirflowDateTimePickerWidget),
-) + FieldConverter.conversion_table
+    *FieldConverter.conversion_table,
+)
 
 
 class UIAlert:
@@ -926,7 +934,7 @@ class UIAlert:
         self.html = html
         self.message = Markup(message) if html else message
 
-    def should_show(self, securitymanager: SecurityManager) -> bool:
+    def should_show(self, appbuilder: AirflowAppBuilder) -> bool:
         """Determine if the user should see the message.
 
         The decision is based on the user's role. If ``AUTH_ROLE_PUBLIC`` is
@@ -934,12 +942,12 @@ class UIAlert:
         ``AUTH_ROLE_PUBLIC`` role.
         """
         if self.roles:
-            current_user = securitymanager.current_user
+            current_user = get_auth_manager().get_user()
             if current_user is not None:
-                user_roles = {r.name for r in securitymanager.current_user.roles}
-            elif "AUTH_ROLE_PUBLIC" in securitymanager.appbuilder.get_app.config:
+                user_roles = {r.name for r in getattr(current_user, "roles", [])}
+            elif "AUTH_ROLE_PUBLIC" in appbuilder.get_app.config:
                 # If the current_user is anonymous, assign AUTH_ROLE_PUBLIC role (if it exists) to them
-                user_roles = {securitymanager.appbuilder.get_app.config["AUTH_ROLE_PUBLIC"]}
+                user_roles = {appbuilder.get_app.config["AUTH_ROLE_PUBLIC"]}
             else:
                 # Unable to obtain user role - default to not showing
                 return False
